// Programmed using the knockoff Pro Micro boards - use Arduino/Genuino Micro as board type
// Watch out - your board's name might change and you'll have to seelct it again in Tools/Port 
// after you program it the first time

#include <Wire.h>

// Libraries 
// You need to install all three libraries - search for them under 
// Sketch/Include Library/Manage Libraries, and then 
// search for the word in front of the .h
#include "Adafruit_MPR121.h"

// Define some bit-wise operations to be used later
#ifndef _BV
#define _BV(bit) (1 << (bit)) 
#endif

// Variables used to calculate centroid value
float centroidArea = 0;
float area = 0;
float prevCentroid = 0;
float currCentroid = 0;

// Initialize the capacitive touch board
Adafruit_MPR121 cap = Adafruit_MPR121();

// Keep track of which pins are touched
uint16_t currtouched = 0;

// A variable used to store initial sensor readings as part of calibration
int cal[11];

// The setup loop will run once to intialize the hardware
void setup() {

  // Wait until the serial port is connected to the serial monitor
  while (!Serial) { // needed to keep leonardo/micro from starting too fast!
    delay(10);
  }
  
  Serial.println("Adafruit MPR121 Capacitive Touch sensor test"); 
  
  // Default address is 0x5A
  if (!cap.begin(0x5A)) {
    Serial.println("MPR121 not found, check wiring?");
    while (1);
  }
  Serial.println("MPR121 found!");
  
  // Wait a little bit before reading each pin and storing a calibration value
  delay(100);
  for(uint8_t i=0; i <= 10; i++){
    cal[i] = cap.filteredData(i);
  }
}


// The main loop repeats over and over...
void loop() {
  // Read from the board to see which pins are touched
  currtouched = cap.touched();

  // Reset variables to zero before doing the calculation
  centroidArea = 0;
  area = 0;
  float f = 0;
  float count = 0;

  // Cycle through each pin, read its value, and add it to the centroid calculation
  for(uint8_t i=0; i <= 10; i++){
    if (currtouched & _BV(i)){
      count++;
      f = cal[i]-cap.filteredData(i);    
    } else {
      f = 0;
    }
    centroidArea = centroidArea + i*f;
    area = area + f; 
  }

  // Calculate a "pressure" value equivalent to how much skin is touching the pad
  float pressure = area/count;

  // Update the new centroid value
  prevCentroid = currCentroid;
  currCentroid = centroidArea/area;

  // If none of the pins are touched, you get a divide by zero error
  // Handle that situation so things don't go wacky!
  if(area == 0){
    currCentroid = prevCentroid;  
    pressure = 0;
  }

  //Print the values to the Serial Monitor
  Serial.println(pressure);
  Serial.println(currCentroid);

  // Centroid ranges from 0-10, so multiply by 10 and then use this in the visualization
  int val = round(10*currCentroid);

  // Print out a visualization of the position measured by the sensor
  Serial.print("I");
  for(uint8_t i=0; i <= 100; i++){
   if (i == val){
    Serial.print("V");
   } else {
    Serial.print("-");    
   }
  }
  Serial.println("I");
  Serial.println("");

  // Wait a bit, because we don't need to have an update more than 10 times a second
  delay(100);
}

// We used code from a library that needs this text included
/*********************************************************
This is a library for the MPR121 12-channel Capacitive touch sensor

Designed specifically to work with the MPR121 Breakout in the Adafruit shop 
  ----> https://www.adafruit.com/products/

These sensors use I2C communicate, at least 2 pins are required 
to interface

Adafruit invests time and resources providing this open source code, 
please support Adafruit and open-source hardware by purchasing 
products from Adafruit!

Written by Limor Fried/Ladyada for Adafruit Industries.  
BSD license, all text above must be included in any redistribution
**********************************************************/
